/**
 * This header file defines all the structures and constants that are used
 * by the SpamPal plugins interface.
 **/

#ifndef _SPAMPAL_H_
#define _SPAMPAL_H_

#include <windows.h>

/**
 * Constant configuration handle used to mean "no configuration loaded" or
 * "configuration does not exist".
 **/

#define NO_CONFIG 0

/**
 * Structure to represent an I.P. address as four octets.
 **/

struct IpAddress {
	unsigned char octets[4];
};


/**
 * Structure to represent a DNSBL service
 **/

struct DnsblList {
	const char *name,
	           *code,
		       *www,
		       *description,
		       *zone;
	const int num_lists_subsumed_in;
	const int* subsumed_in;  // indexes of the lists this one is subsumed in
	const int num_result_codes;
	const struct IpAddress* result_codes;  // the allowed result codes
									 // if none, then any result allowed
};



/**
 * The types of reason that can appear in the X-SpamPal: header
 **/

#define REASON_TYPE_NONE 0
#define REASON_TYPE_HEAD 1
#define REASON_TYPE_BODY 2
#define REASON_TYPE_TIME_OUT 3
#define REASON_TYPE_OTHER 4

/**
 * The reasons that can appear in the X-SpamPal: header
 **/

#define REASON_NONE 0
#define REASON_BLIST_FROM 1
#define REASON_BLIST_IP 2
#define REASON_WLIST_FROM 3
#define REASON_WLIST_IP 4
#define REASON_AWLIST_FROM 5
#define REASON_AWLIST_IP 6
#define REASON_DNSBL 7
#define REASON_OTHER 8

/**
 * Encapsulates information to be used in X-SpamPal header
 */

struct xSpamPalReason {
	int type;  // REASON_TYPE_HEAD or BODY or NONE
	int reason;  // REASON_NONE or REASON_DNSBL or etc
	char otherReason[5];  // only used if REASON=REASON_TYPE_OTHER
	union {
		struct {
			const struct DnsblList* dnsblList; // if reason=REASON_DNSBL; NULL otherwise
			struct IpAddress ipAddr;
		} ipBlock;  // if reason==REASON_DNSBL or REASON_*_IP
		struct {
			char description[24];
		} unknown;  // if reason=REASON_OTHER
	} details;
};


/**
 * Encapsulates the status of a given email message.
 **/

// Information about the session
#define SPAMPAL_CONNECTION_POP3 1
#define SPAMPAL_CONNECTION_IMAP4 2
struct SessionInfo {
	char* serverName;
	int serverType;
	char* userName;
};


// Information about the current message
struct MessageInfo {
	unsigned long size;
	BOOL isComplete;
};


// Information about the status of an email message.
struct MessageStatus {
	BOOL isSpam, isWhitelisted, forceBodyFiltering;
	struct xSpamPalReason xSpamPalHeader;
    const struct SessionInfo* sessionInfo;
    const struct MessageInfo* msgInfo;
};




/**
 * Various methods in the SpamPal executable that can be called from the
 * plugin's interface.
 **/

struct SpamPalFilteringHooks {
	const void (*filterOnEmailAddress) (
		unsigned long fetchHandle,
		const char* email,
		BOOL awList,
		int reasonType,
		const char* reasonTypeOther
	);
	const void (*filterOnIpAddress) (
		unsigned long fetchHandle,
		const struct IpAddress* addr,
		BOOL awList,
		int reasonType,
		const char* reasonTypeOther
	);
	const void (*filterOnHost) (
		unsigned long fetchHandle,
		const char* hostname,
		BOOL awList,
		int reasonType,
		const char* reasonTypeOther
	);
	const void (*filterOnRawDnsbl) (
		unsigned long fetchHandle,
		const char* dnsblQuery,
		const struct IpAddress* positiveResultCodes,
		int numPositiveResultCodes,
		int queryType,
		const struct xSpamPalReason* reason
	);
	const int (*findEmailAddress) (
		const char* str,
		char* buf
	);
	const int (*findDottedQuad) (
		const char* str,
		struct IpAddress *ipAddr
	);
};

// Types of "raw" DNSBL query
#define QUERY_TYPE_SPAM 0
#define QUERY_TYPE_WHITELIST 1

// Possible sources of email addresses
#define ADDRESS_SOURCE_FROM 1
#define ADDRESS_SOURCE_SENDER 2
#define ADDRESS_SOURCE_REPLYTO 3
#define ADDRESS_SOURCE_PLUGIN 128


#endif

